@@ -41,11 +41,13 @@ if 1:
     try:
         from matplotlib import pyplot as plt
         plt.imshow(M, aspect='equal')
-        plt.spectral()
         try:
-            import sys, signal
-            def action(*args):
-                raise SystemExit
+            plt.nipy_spectral()
+        except AttributeError:
+            plt.spectral()
+        try:
+            import signal
+            def action(*args): raise SystemExit
             signal.signal(signal.SIGALRM, action)
             signal.alarm(2)
         except:
