@@ -2,11 +2,8 @@ from mpi4py import MPI
 import mpiunittest as unittest
 import arrayimpl
 
-try:
-    _reduce = reduce
-except NameError:
-    from functools import reduce as _reduce
-prod = lambda sequence,start=1: _reduce(lambda x, y: x*y, sequence, start)
+from functools import reduce
+prod = lambda sequence,start=1: reduce(lambda x, y: x*y, sequence, start)
 
 def maxvalue(a):
     try:
@@ -174,11 +171,11 @@ class BaseTestCCOBuf(object):
                         if op == MPI.SUM:
                             redval = sum(range(size))+size
                             if redval < max_val:
-                                self.assertEqual(value, redval)
+                                self.assertAlmostEqual(value, redval)
                         elif op == MPI.PROD:
                             redval = prod(range(1,size+1))
                             if redval < max_val:
-                                self.assertEqual(value, redval)
+                                self.assertAlmostEqual(value, redval)
                         elif op == MPI.MAX:
                             self.assertEqual(value, size)
                         elif op == MPI.MIN:
@@ -192,11 +189,11 @@ class BaseTestCCOBuf(object):
                         if op == MPI.SUM:
                             redval = sum(range(size))+size
                             if redval < max_val:
-                                self.assertEqual(value, redval)
+                                self.assertAlmostEqual(value, redval)
                         elif op == MPI.PROD:
                             redval = prod(range(1,size+1))
                             if redval < max_val:
-                                self.assertEqual(value, redval)
+                                self.assertAlmostEqual(value, redval)
                         elif op == MPI.MAX:
                             self.assertEqual(value, size)
                         elif op == MPI.MIN:
@@ -230,9 +227,9 @@ class BaseTestCCOBuf(object):
                                 if v_prod <= max_val:
                                     self.assertAlmostEqual(value, v_prod)
                             elif op == MPI.MAX:
-                                self.assertAlmostEqual(value, v_max)
+                                self.assertEqual(value, v_max)
                             elif op == MPI.MIN:
-                                self.assertAlmostEqual(value, v_min)
+                                self.assertEqual(value, v_min)
 
     def testScan(self):
         size = self.COMM.Get_size()
@@ -272,7 +269,7 @@ class BaseTestCCOBuf(object):
                                          rbuf.as_mpi(),
                                          op)
                     except NotImplementedError:
-                        return
+                        self.skipTest('mpi-exscan')
                     if rank == 1:
                         for i, value in enumerate(rbuf):
                             self.assertEqual(value, i)
@@ -290,7 +287,6 @@ class BaseTestCCOBuf(object):
                             elif op == MPI.MIN:
                                 self.assertEqual(value, i)
 
-
     def testBcastTypeIndexed(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
@@ -299,10 +295,10 @@ class BaseTestCCOBuf(object):
                 for root in range(size):
                     #
                     if rank == root:
-                        buf = array(range(10), typecode)
+                        buf = array(range(10), typecode).as_raw()
                     else:
-                        buf = array(-1, typecode, 10)
-                    indices = range(0, len(buf), 2)
+                        buf = array(-1, typecode, 10).as_raw()
+                    indices = list(range(0, len(buf), 2))
                     newtype = datatype.Create_indexed_block(1, indices)
                     newtype.Commit()
                     newbuf = (buf, 1, newtype)
@@ -317,10 +313,10 @@ class BaseTestCCOBuf(object):
 
                     #
                     if rank == root:
-                        buf = array(range(10), typecode)
+                        buf = array(range(10), typecode).as_raw()
                     else:
-                        buf = array(-1, typecode, 10)
-                    indices = range(1, len(buf), 2)
+                        buf = array(-1, typecode, 10).as_raw()
+                    indices = list(range(1, len(buf), 2))
                     newtype = datatype.Create_indexed_block(1, indices)
                     newtype.Commit()
                     newbuf = (buf, 1, newtype)
@@ -346,20 +342,20 @@ class BaseTestCCOBufInplace(object):
                     if rank == root:
                         sbuf = MPI.IN_PLACE
                         buf = array(-1, typecode, (size, count))
-                        buf.flat[(rank*count):((rank+1)*count)] = \
-                            array(root, typecode, count)
+                        #buf.flat[(rank*count):((rank+1)*count)] = \
+                        #    array(root, typecode, count)
+                        s, e = rank*count, (rank+1)*count
+                        for i in range(s, e): buf.flat[i] = root
                         rbuf = buf.as_mpi()
                     else:
                         buf = array(root, typecode, count)
                         sbuf = buf.as_mpi()
                         rbuf = None
-                    try:
-                        self.COMM.Gather(sbuf, rbuf, root=root)
-                    except NotImplementedError:
-                        return
+                    self.COMM.Gather(sbuf, rbuf, root=root)
                     for value in buf.flat:
                         self.assertEqual(value, root)
 
+    @unittest.skipMPI('msmpi(==10.0.0)')
     def testScatter(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
@@ -375,10 +371,7 @@ class BaseTestCCOBufInplace(object):
                             buf = array(-1, typecode, count)
                             sbuf = None
                             rbuf = buf.as_mpi()
-                        try:
-                            self.COMM.Scatter(sbuf, rbuf, root=root)
-                        except NotImplementedError:
-                            return
+                        self.COMM.Scatter(sbuf, rbuf, root=root)
                         for value in buf.flat:
                             self.assertEqual(value, root)
 
@@ -389,12 +382,11 @@ class BaseTestCCOBufInplace(object):
             for typecode in arrayimpl.TypeMap:
                 for count in range(1, 10):
                     buf = array(-1, typecode, (size, count))
-                    buf.flat[(rank*count):((rank+1)*count)] = \
-                        array(count, typecode, count)
-                    try:
-                        self.COMM.Allgather(MPI.IN_PLACE, buf.as_mpi())
-                    except NotImplementedError:
-                        return
+                    #buf.flat[(rank*count):((rank+1)*count)] = \
+                    #    array(count, typecode, count)
+                    s, e = rank*count, (rank+1)*count
+                    for i in range(s, e): buf.flat[i] = count
+                    self.COMM.Allgather(MPI.IN_PLACE, buf.as_mpi())
                     for value in buf.flat:
                         self.assertEqual(value, count)
 
@@ -421,10 +413,7 @@ class BaseTestCCOBufInplace(object):
                             buf2 = array(range(size), typecode)
                             sbuf = buf.as_mpi()
                             rbuf = buf2.as_mpi()
-                        try:
-                            self.COMM.Reduce(sbuf, rbuf, op, root)
-                        except NotImplementedError:
-                            return
+                        self.COMM.Reduce(sbuf, rbuf, op, root)
                         if rank == root:
                             max_val = maxvalue(buf)
                             for i, value in enumerate(buf):
@@ -487,16 +476,17 @@ class BaseTestCCOBufInplace(object):
                                 if op == MPI.SUM:
                                     redval = sum(range(size))
                                     if redval < max_val:
-                                        self.assertEqual(value, redval)
+                                        self.assertAlmostEqual(value, redval)
                                 elif op == MPI.PROD:
                                     redval = prod(range(1,size+1))
                                     if redval < max_val:
-                                        self.assertEqual(value, redval)
+                                        self.assertAlmostEqual(value, redval)
                                 elif op == MPI.MAX:
                                     self.assertEqual(value, size-1)
                                 elif op == MPI.MIN:
                                     self.assertEqual(value, 0)
 
+    @unittest.skipMPI('MVAPICH2')
     def testReduceScatter(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
@@ -522,16 +512,107 @@ class BaseTestCCOBufInplace(object):
                             if op == MPI.SUM:
                                 redval = sum(range(size))
                                 if redval < max_val:
-                                    self.assertEqual(value, redval)
+                                    self.assertAlmostEqual(value, redval)
                             elif op == MPI.PROD:
                                 redval = prod(range(1,size+1))
                                 if redval < max_val:
-                                    self.assertEqual(value, redval)
+                                    self.assertAlmostEqual(value, redval)
                             elif op == MPI.MAX:
                                 self.assertEqual(value, size-1)
                             elif op == MPI.MIN:
                                 self.assertEqual(value, 0)
 
+    @unittest.skipMPI('openmpi(<=1.8.4)')
+    def testScan(self):
+        size = self.COMM.Get_size()
+        rank = self.COMM.Get_rank()
+        # --
+        for array in arrayimpl.ArrayTypes:
+            for typecode in arrayimpl.TypeMap:
+                for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
+                    buf = array(range(size), typecode)
+                    self.COMM.Scan(MPI.IN_PLACE,
+                                   buf.as_mpi(),
+                                   op)
+                    max_val = maxvalue(buf)
+                    for i, value in enumerate(buf):
+                        if op == MPI.SUM:
+                            if (i * (rank + 1)) < max_val:
+                                self.assertAlmostEqual(value, i * (rank + 1))
+                        elif op == MPI.PROD:
+                            if (i ** (rank + 1)) < max_val:
+                                self.assertAlmostEqual(value, i ** (rank + 1))
+                        elif op == MPI.MAX:
+                            self.assertEqual(value, i)
+                        elif op == MPI.MIN:
+                            self.assertEqual(value, i)
+
+    @unittest.skipMPI('openmpi(<=1.8.4)')
+    @unittest.skipMPI('msmpi(<=4.2.0)')
+    def testExscan(self):
+        size = self.COMM.Get_size()
+        rank = self.COMM.Get_rank()
+        for array in arrayimpl.ArrayTypes:
+            for typecode in arrayimpl.TypeMap:
+                for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
+                    buf = array(range(size), typecode)
+                    try:
+                        self.COMM.Exscan(MPI.IN_PLACE,
+                                         buf.as_mpi(),
+                                         op)
+                    except NotImplementedError:
+                        self.skipTest('mpi-exscan')
+                    if rank == 1:
+                        for i, value in enumerate(buf):
+                            self.assertEqual(value, i)
+                    elif rank > 1:
+                        max_val = maxvalue(buf)
+                        for i, value in enumerate(buf):
+                            if op == MPI.SUM:
+                                if (i * rank) < max_val:
+                                    self.assertAlmostEqual(value, i * rank)
+                            elif op == MPI.PROD:
+                                if (i ** rank) < max_val:
+                                    self.assertAlmostEqual(value, i ** rank)
+                            elif op == MPI.MAX:
+                                self.assertEqual(value, i)
+                            elif op == MPI.MIN:
+                                self.assertEqual(value, i)
+
+class TestReduceLocal(unittest.TestCase):
+
+    def testReduceLocal(self):
+        for array in arrayimpl.ArrayTypes:
+            for typecode in arrayimpl.TypeMap:
+                for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
+                    size = 5
+                    sbuf = array(range(1,size+1), typecode)
+                    rbuf = array(range(0,size+0), typecode)
+                    try:
+                        op.Reduce_local(sbuf.as_mpi(), rbuf.as_mpi())
+                    except NotImplementedError:
+                        self.skipTest('mpi-op-reduce_local')
+                    for i, value in enumerate(rbuf):
+                        self.assertEqual(sbuf[i], i+1)
+                        if op == MPI.SUM:
+                            self.assertAlmostEqual(value, i+(i+1))
+                        elif op == MPI.PROD:
+                            self.assertAlmostEqual(value, i*(i+1))
+                        elif op == MPI.MAX:
+                            self.assertEqual(value, i+1)
+                        elif op == MPI.MIN:
+                            self.assertEqual(value, i)
+        for array in arrayimpl.ArrayTypes:
+            for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
+                sbuf = array(range(3), "i")
+                rbuf = array(range(3), "i")
+                def f(): op.Reduce_local(sbuf.as_mpi_c(2),
+                                         rbuf.as_mpi_c(3))
+                self.assertRaises(ValueError, f)
+                def f(): op.Reduce_local([sbuf.as_raw(), 1, MPI.INT],
+                                         [rbuf.as_raw(), 1, MPI.SHORT])
+                self.assertRaises(ValueError, f)
+
 
 class TestCCOBufSelf(BaseTestCCOBuf, unittest.TestCase):
     COMM = MPI.COMM_SELF
@@ -539,36 +620,34 @@ class TestCCOBufSelf(BaseTestCCOBuf, unittest.TestCase):
 class TestCCOBufWorld(BaseTestCCOBuf, unittest.TestCase):
     COMM = MPI.COMM_WORLD
 
+@unittest.skipMPI('MPICH1')
+@unittest.skipMPI('LAM/MPI')
+@unittest.skipIf(MPI.IN_PLACE == MPI.BOTTOM, 'mpi-in-place')
 class TestCCOBufInplaceSelf(BaseTestCCOBufInplace, unittest.TestCase):
     COMM = MPI.COMM_SELF
 
+@unittest.skipMPI('MPICH1')
+@unittest.skipMPI('LAM/MPI')
+@unittest.skipIf(MPI.IN_PLACE == MPI.BOTTOM, 'mpi-in-place')
 class TestCCOBufInplaceWorld(BaseTestCCOBufInplace, unittest.TestCase):
     COMM = MPI.COMM_WORLD
+    @unittest.skipMPI('IntelMPI', MPI.COMM_WORLD.Get_size() > 1)
+    def testReduceScatter(self):
+        super(TestCCOBufInplaceWorld, self).testReduceScatter()
 
-class TestCCOBufSelfDup(BaseTestCCOBuf, unittest.TestCase):
+class TestCCOBufSelfDup(TestCCOBufSelf):
     def setUp(self):
         self.COMM = MPI.COMM_SELF.Dup()
     def tearDown(self):
         self.COMM.Free()
 
-class TestCCOBufWorldDup(BaseTestCCOBuf, unittest.TestCase):
+@unittest.skipMPI('openmpi(<1.4.0)', MPI.Query_thread() > MPI.THREAD_SINGLE)
+class TestCCOBufWorldDup(TestCCOBufWorld):
     def setUp(self):
         self.COMM = MPI.COMM_WORLD.Dup()
     def tearDown(self):
         self.COMM.Free()
 
 
-_name, _version = MPI.get_vendor()
-if (_name == 'MPICH1' or _name == 'LAM/MPI'
-    or MPI.BOTTOM == MPI.IN_PLACE):
-    del BaseTestCCOBufInplace
-    del TestCCOBufInplaceSelf
-    del TestCCOBufInplaceWorld
-elif _name == 'Open MPI':
-    if _version < (1, 4, 0):
-        if MPI.Query_thread() > MPI.THREAD_SINGLE:
-            del TestCCOBufWorldDup
-
-
 if __name__ == '__main__':
     unittest.main()
